window.boot = function () {
    var settings = window._CCSettings;
    window._CCSettings = undefined;

    if (!settings.debug) {
        var uuids = settings.uuids;

        var rawAssets = settings.rawAssets;
        var assetTypes = settings.assetTypes;
        var realRawAssets = settings.rawAssets = {};
        for (var mount in rawAssets) {
            var entries = rawAssets[mount];
            var realEntries = realRawAssets[mount] = {};
            for (var id in entries) {
                var entry = entries[id];
                var type = entry[1];
                // retrieve minified raw asset
                if (typeof type === 'number') {
                    entry[1] = assetTypes[type];
                }
                // retrieve uuid
                realEntries[uuids[id] || id] = entry;
            }
        }

        var scenes = settings.scenes;
        for (var i = 0; i < scenes.length; ++i) {
            var scene = scenes[i];
            if (typeof scene.uuid === 'number') {
                scene.uuid = uuids[scene.uuid];
            }
        }

        var packedAssets = settings.packedAssets;
        for (var packId in packedAssets) {
            var packedIds = packedAssets[packId];
            for (var j = 0; j < packedIds.length; ++j) {
                if (typeof packedIds[j] === 'number') {
                    packedIds[j] = uuids[packedIds[j]];
                }
            }
        }
    }

    var onStart = function () {
        cc.loader.downloader._subpackages = settings.subpackages;

        cc.view.enableRetina(true);
        cc.view.resizeWithBrowserSize(true);

        // init assets
        cc.AssetLibrary.init({
            libraryPath: 'res/import',
            rawAssetsBase: 'res/raw-',
            rawAssets: settings.rawAssets,
            packedAssets: settings.packedAssets,
            md5AssetsMap: settings.md5AssetsMap
        });

        var launchScene = settings.launchScene;

        // load scene
        cc.director.loadScene(launchScene, null,
            function () {
                cc.loader.onProgress = null;
                console.log('Success to load scene: ' + launchScene);
            }
        );
    };

    // jsList
    var jsList = settings.jsList;
    var bundledScript = settings.debug ? 'src/project.dev.js' : 'src/project.js';
    if (jsList) {
        jsList = jsList.map(function (x) {
            return 'src/' + x;
        });
        jsList.push(bundledScript);
    }
    else {
        jsList = [bundledScript];
    }

    // anysdk scripts
    if (cc.sys.isNative && cc.sys.isMobile) {
        jsList = jsList.concat(['src/anysdk/jsb_anysdk.js', 'src/anysdk/jsb_anysdk_constants.js']);
    }

    cc.renderer.renderEngine.gfx.VertexBuffer.prototype.update = function update(offset, data) {
        if (this._glID === -1) {
            console.error('The buffer is destroyed');
            return;
        }

        if (data && data.byteLength + offset > this._bytes) {
            if (offset) {
                console.error('2_Failed to update data, bytes exceed.');
                return;
            } else {
                this._bytes = offset + data.byteLength;
                this.numVertices = this._bytes / this._format._bytes;
            }
        }

        var gl = this._device._gl;
        var glUsage = this._usage;

        gl.bindBuffer(gl.ARRAY_BUFFER, this._glID);
        if (!data) {
            if (this._bytes) {
                gl.bufferData(gl.ARRAY_BUFFER, this._bytes, glUsage);
            } else {
                console.warn('bufferData should not submit 0 bytes data');
            }
        } else {
            if (offset) {
                gl.bufferSubData(gl.ARRAY_BUFFER, offset, data);
            } else {
                gl.bufferData(gl.ARRAY_BUFFER, data, glUsage);
            }
        }
        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    };

    cc.renderer.renderEngine.gfx.IndexBuffer.prototype.update = function update(offset, data) {
        if (this._glID === -1) {
            console.error('The buffer is destroyed');
            return;
        }

        if (data && data.byteLength + offset > this._bytes) {
            if (offset) {
                console.error('1_Failed to update data, bytes exceed.');
                return;
            } else {
                this._bytes = offset + data.byteLength;
                this.numIndices = this._bytes / this._bytesPerIndex;
            }
        }

        var gl = this._device._gl;
        var glUsage = this._usage;

        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this._glID);
        if (!data) {
            if (this._bytes) {
                gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, this._bytes, glUsage);
            } else {
                console.warn('bufferData should not submit 0 bytes data');
            }
        } else {
            if (offset) {
                gl.bufferSubData(gl.ELEMENT_ARRAY_BUFFER, offset, data);
            } else {
                gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, data, glUsage);
            }
        }
        this._device._restoreIndexBuffer();
    };

    var option = {
        id: 'GameCanvas',
        scenes: settings.scenes,
        debugMode: settings.debug ? cc.debug.DebugMode.INFO : cc.debug.DebugMode.ERROR,
        showFPS: settings.debug,
        frameRate: 60,
        jsList: jsList,
        groupList: settings.groupList,
        collisionMatrix: settings.collisionMatrix,
        renderMode: 0
    }

    cc.preDefined = {};
    cc.preDefined.type = 1;
    require('predefined.js');
    console.log('cc.preDefined = ' + JSON.stringify(cc.preDefined));

    cc.game.run(option, onStart);
};

if (jsb) {
    var hotUpdateSearchPaths = localStorage.getItem('HotUpdateSearchPaths');
    if (hotUpdateSearchPaths) {
        jsb.fileUtils.setSearchPaths(JSON.parse(hotUpdateSearchPaths));
    }
}

require('src/settings.js');
require('src/cocos2d-jsb.js');
require('jsb-adapter/jsb-engine.js');
window.boot();
